import os
import sys
import cgi
from unidecode import unidecode
import zipfile
import zlib
import string
import re
from datetime import datetime
from pathlib import Path
# Этот скрипт архивирует все .docx и .fb2 файлы в рабочей директории, преобразуя их имена в транслит,
# после чего генерирует файл list.txt со строками для файла конфигурации, включающими все архивированные
# тексты. Для нормальной работы скрипта необходимо, чтобы были соблюдены все правила именования книг,
# а все файлы .docx были укомплектованы .fb2 с посимвольным совпадением имён. Названия текстов
# автоматически модифицируются с переводом специальных символов в HTML entities. Параметр dat выставляется
# по времени срабатывания скрипта. В отличие от одноимённого скрипта без постфикса _new этот записывает
# в начало файла конфигурации шлавную строку конфига и псевдотег-заготовку для биографии.
path = os.getcwd()
os.chdir(path)
files = []
archives = []
my_list = "list.txt"
# Читаем все файлы из рабочей директории
for file in os.listdir(path):
    if file.endswith('.fb2') or file.endswith('.docx'):
        files.append(file)

# Выходим, если книг, подлежащих архивации, в директории нет
if not files:
    sys.exit()

# Если они есть, то транслитерируем названия файлов и формируем имена архивов
names_only = []
for book in files:
    book_full = os.path.join(path, book)
    book_name, book_extension = os.path.splitext(book)
    names_only.append(book_name)
    if book_extension == ".docx":
        book_extension = ".zip"
    else:
        book_extension = ".fb2.zip"
    book_name = re.sub('^.*? -- ', '', book_name)
    book_name = re.sub(
        r'[\,\:\;\!\¡\?\¿\°«»„“”‘’\'\"「」『』‹›\<\>《》\(\)\[\]_\-–—\\¤\#\@\$\%\^\&\*\+\|\{\}]',
        ' ', book_name)
    book_name = re.sub(' +', '_', book_name)
    book_name = unidecode(book_name)
    book_name = re.sub(' +', '_', book_name)
    book_name = re.sub('_+', '_', book_name)
    book_name = re.sub('\'', '', book_name)
    archbook = book_name + book_extension
    archives.append(archbook)

# Архивируем все книги
compression = zipfile.ZIP_DEFLATED
for book, archive in zip(files, archives):
    zipfile.ZipFile(archive, "w", compression).write(book)
    os.remove(os.path.join(path, book))

# Формируем строку текущей даты в формате YYMMDD
timestamp = datetime.now()
cur_date = timestamp.strftime("%y%m%d")

# Формируем шаблон записи для файла конфигурации
record_source = "<book lvl=\"0\" dat=\"#dat#\" vis=\"1\" lng=\"#lng#\" nam=\"#nam#\" lnk=\"#lnk#\">"

# Вставляем текущую дату
record_source = re.sub('#dat#', cur_date, record_source)


# Задаём функцию преобразования специальных символов в HTML entities
def myord(m):
    return "&#%s;" % ord(m.group())


# Заполняем массив записей для list.txt
records = []
for i in range(0, len(files), 2):
    rec = record_source
    title = names_only[i]
    title = re.sub('^.*? -- ', '', title)
    lng = "RUS"
    m = re.search(r'\.([A-Z]{3}$)', title)
    if m:
        lng = m.group(1)
        title = re.sub(r'(\.[A-Z]{3}$)', '', title)
    title = re.sub(
        r'[^ \.\,\?\¿\!\¡\:\;「」『』‹›\<\>《》\(\)\[\]0-9A-Za-zА-ЯЁа-яё]', myord,
        title)
    rec = re.sub('#lng#', lng, rec)
    rec = re.sub('#nam#', title, rec)
    link = archives[i + 1] + "|" + archives[i]
    rec = re.sub('#lnk#', link, rec)
    records.append(rec)

# Убеждаемся, что файла list.txt нет в рабочей директории, если он существует --
# добавляем единичку к имени и проверяем снова до достижения уникальности
my_file = Path(my_list)
while my_file.is_file():
    list_name, list_extension = os.path.splitext(my_list)
    list_name = list_name + "1"
    my_list = list_name + list_extension
    my_file = Path(my_list)

# Печатаем конфигурационные строки в list.txt
f = open(my_list, "w")
f.write(
    "<file nam=\"Автор¤Авторов\" aon=\"\" lng=\"RUS\" vis=\"1\" gnr=\"0\">" +
    "\n")
f.write("<text></text>" + "\n")
for rec in records:
    f.write(rec + "\n")
f.close()