import os
import sys
import string
import re
# Валидатор конфигов библиотеки. Проверяет следующее:
#    1. Чётность числа кавычек (все открытые кавычки д. б. закрыты).
#    2. Легитимность символов в биографии автора.
#    3. Легитимность символов в параметре nam книжных тегов.
#    4. Проверка существования архивов.
#    5. Присутствие кода языка в названии архивов при наличии его в параметре lng книги.
# Результаты пишутся в файл error_log.txt.
# Не забываем прописывать правильные пути к архивам и конфигу (переменные lib_cfg_path и lib_arc_path в строках 20-21)!
# Для проверки правильности ссылок на архивы последние должны находиться в папках, соответствующих названиям файлов
# конфигов (напр. /abe для конфига abe.txt), расположенных по адресу, задаваемому lib_arc_path.
path = os.getcwd()
os.chdir(path)
cfg_files = []
errors = []
error_log = "error_log.txt"
#lib_arc_path = "F:\\Alex\\Scripts"
lib_cfg_path = "F:\\Alex\\Scripts"
lib_arc_path = "D:\\!!!Work\\Draft_Webs\\Artefact\\library\\books"


# Функция дополнения номера строки нулями
def addzeros(s):
    while len(s) < 3:
        s = "0" + s
    return s


# Проверка наличия в биографии неправильных символов
def checkbiosmb(ntc, ltc):
    m = re.search(r'<text>(.*?)</text>', ltc)
    if m:
        nam = m.group(1)
        regexp = re.compile(r'[¿°«»„“”‘’「」『』‹›《》–—\\\$\^\*\+\|\{\}]+')
        if regexp.search(nam):
            stpos = regexp.search(nam).start()
            errors.append("Строка " + addzeros(ntc) +
                          ": неверный символ в позиции " + str(stpos + 1) +
                          ": [" + nam[stpos] + "]")


# Проверка наличия кода языка в названии архива при наличии его в параметре lng
def checklng(ntc, ltc):
    m = re.search(r'lng=\"(.*?)\"', ltc)
    n = re.search(r'lnk=\"(.+?)\"', ltc)
    b = re.search(r'nam=\"(.*?)\"', ltc)
    if m and n:
        lng = m.group(1)
        if lng != "RUS":
            lnk = n.group(1)
            links = lnk.split("|")
            for k in range(len(links)):
                lng_code = "." + lng + "."
                if lng_code not in links[k]:
                    errors.append("Строка " + addzeros(ntc) +
                                  ": код языка отсутствует в ссылке [" +
                                  links[k] + "] на книгу [" + b.group(1) + "]")


# Проверка наличия архивов по ссылке (ссылкам)
def checklinks(ntc, ltc):
    m = re.search(r'lnk=\"(.+?)\"', ltc)
    if m:
        lnk = m.group(1)
        links = lnk.split("|")
        for k in range(len(links)):
            cname = os.path.join(lib_arc_path, alias, links[k])
            if not os.path.exists(cname):
                errors.append("Строка " + addzeros(ntc) +
                              ": файл не найден [" + cname + "]")


# Проверка чётности числа кавычек
def checkquotes(ntc, ltc):
    q_count = ltc.count("\"")
    if q_count / 2 != int(q_count / 2):
        errors.append("Строка " + addzeros(ntc) + " : непарные кавычки. [" +
                      ltc + "]")


# Проверка наличия в строке неправильных символов
def checksmb(ntc, ltc):
    m = re.search(r'nam=\"(.*?)\"', ltc)
    if m:
        nam = m.group(1)
        regexp = re.compile(
            r'[¿°«»„“”‘’\'\"「」『』‹›\<\>《》\(\)\[\]_–—\\\$\@\%\^\*\+\|\{\}]+')
        if regexp.search(nam):
            stpos = regexp.search(nam).start()
            errors.append("Строка " + addzeros(ntc) +
                          ": неверныйй символ в параметре NAM в позиции " +
                          str(stpos + 1) + ": [" + nam[stpos] + "]")


# Функция обработки строк
def checkline(n, s):
    ntc = str(n + 1)
    if "file nam" in s or "group vis" in s or "book lvl" in s:  # Проверяем символы в nam у всех подходящих строк
        checkquotes(str(ntc), s)
        checksmb(ntc, s)
    if "book lvl" in s:  # Проверяем наличие кода языка в названии архива при наличии его в параметре lng
        checklng(ntc, s)
    if "<text>" in s:  # Проверяем символы в биографии
        checkquotes(ntc, s)
        checkbiosmb(ntc, s)
    if "lnk=" in s:  # Проверяем наличие архивов по ссылке (ссылкам)
        checklinks(ntc, s)


# Читаем все файлы .txt
for cfg_file in os.listdir(lib_cfg_path):
    if cfg_file.endswith('.txt'):
        cfg_files.append(cfg_file)

# Выходим, если файлов .txt в директории нет
if not cfg_files:
    sys.exit()

cfg_contents = []
# Если они есть, то читаем все
for cfg_file in cfg_files:
    f = open(os.path.join(lib_cfg_path, cfg_file), 'r')
    contents = f.read()
    cfg_contents.append(contents)
    f.close()

# Режем конфиг на строки
f = open(error_log, "w", encoding="utf-8")
for i in range(len(cfg_contents)):
    errors.clear()
    m = re.search(r'file nam=\"(.*?)\"', cfg_contents[i])
    if m:
        anam = m.group(1)
    else:
        anam = "Без имени автора"
    cfg_lines = cfg_contents[i].split('\n')
    # Перебираем полученные строки
    for j in range(len(cfg_lines)):
        alias, _ = os.path.splitext(cfg_files[i])
        checkline(j, cfg_lines[j])
    if errors:
        n1, n2 = anam.split("¤")
        nam = n2 + ", " + n1
        f.write("\n" + nam + " (" + cfg_files[i] + ")" + "\n")
        f.write("-------------------------------------------------" + "\n")
        for error in errors:
            f.write(error + "\n")
f.close()
